# -*- coding: utf-8 -*-
import objectpack
from m3_ext.ui.results import ExtUIComponentResult
from .ui import BaseGridWindow


class BaseGridPack(objectpack.BasePack):
    u"""
    Пак умеющий строить динамический грид с изменяющимся
    количеством колонок в зависимости от значений полей фильтрации.

    """
    window = BaseGridWindow

    # аттрибуты грида
    column_param_name = None
    u"""
    Например, для колонок из классов можно определить

    column_param_name = 'classyear_id'
    """
    row_id_name = None
    u"""
    Например, для строк из звонков расписания можно определить

    row_id_name = 'call_id'
    """

    @property
    def id_param_name(self):
        return '%s_id' % self.short_name

    @property
    def grid_panel(cls):
        u"""
        Класс панели с гридом.

        .. note::
            Вынесено в метод для случая, когда пак работает с гридом,
            размещенным не в окне, а, например, в табе
        """
        return cls.window.grid_panel_cls

    def __init__(self):
        super(BaseGridPack, self).__init__()
        self.grid_action = BaseGridCreateAction()
        self.window_action = BaseGridWinAction()
        self.actions.extend([self.grid_action, self.window_action])

    def create_columns(self, request, context):
        u"""
        Метод возвращающий колонки.

        .. seealso:: objectpack.actions.ObjectPack.columns

        :rtype: list
        """
        return []

    def create_grid(self, columns):
        u"""
        Создание грида.

        :param list columns: список колонок
        :rtype: m3_ext.ui.ExtObjectGrid
        """
        grid = self.grid_panel.create_grid(pack=self, columns=columns)
        return grid

    def set_grid_params(self, grid, request, context):
        u"""
        Настройка параметров для конфигурации грида.

        Параметры грида передаются панели с гридом в метод configure_grid.
        :param grid: грид
        :type grid: m3_ext.ui.ExtObjectGrid
        :param request: Запрос
        :type request: django.http.HttpRequest
        :param context: Контекст
        :type context: m3.actions.context.DeclarativeActionContext
        :rtype: dict
        :returns: словарь параметров для создания грида
        """
        # пак для настройки грида (column_param_name, id_param_name, url_data)
        params = {'pack': self}
        return params

    def configure_grid(self, grid, request, context):
        u"""
        Метод конфигурирования грида после его создания.
        :param grid: грид
        :type grid: m3_ext.ui.ExtObjectGrid
        :param request: Запрос
        :type request: django.http.HttpRequest
        :param context: Контекст
        :type context: m3.actions.context.DeclarativeActionContext
        """
        params = self.set_grid_params(grid, request, context)
        grid = self.grid_panel.configure_grid(grid, params)
        return grid

    def get_grid_action_url(self):
        u"""
        Получение адреса экшна построения грид.

        :rtype: str
        """
        return self.grid_action.get_absolute_url()

    def get_rows_url(self):
        u"""
        Получение адреса экшна, возвращающего строки грида.

        Должен быть определен в потомке.
        :rtype: str
        """
        raise NotImplementedError(
            u"Не определен метод get_rows_url() "
            u"в %s!" % self.__class__.__name__)

    def create_window(self, request, context):
        u"""Получение окна."""
        return self.window()

    def get_window_params(self, request, context):
        u"""
        Параметры для показа окна
        :param request: Запрос
        :type request: django.http.HttpRequest
        :param context: Контекст
        :type context: m3.actions.context.DeclarativeActionContext
        :rtype: dict

        .. note::
            Используется только если данный пак отвечает и за показ окна
            с гридом

        """
        params = {
            # пак отвечающий за получения грида окном
            "grid_pack": self
        }
        return params


class BaseGridCreateAction(objectpack.BaseAction):
    u"""Базовый экшн построения грида."""

    perm_code = 'view'

    def run(self, request, context):
        pack = self.parent
        columns = pack.create_columns(request, context)
        grid = pack.create_grid(columns)
        pack.configure_grid(grid, request, context)
        return ExtUIComponentResult(grid)


class BaseGridWinAction(objectpack.BaseWindowAction):
    u"""Экшн показа окна."""

    perm_code = 'view'

    def create_window(self):
        self.win = self.parent.create_window(self.request, self.context)

    def set_window_params(self):
        super(BaseGridWinAction, self).set_window_params()
        self.win_params = self.parent.get_window_params(
            self.request, self.context)
